@@ -25,12 +25,14 @@ from .data import (
     ListData,
     data_attribute,
     data_dict_attribute,
-    data_list_attribute,
+    data_protected_dict_attribute,
+    data_protected_list_attribute,
     data_set_attribute,
 )
 from .utils.custom_repr import custom_mapping_repr
 from .utils.recursive_repr import recursive_repr
 from .utils.reraise_context import ReraiseContext
+from .utils.storage import Storage
 from .utils.type_checking import (
     assert_is_callable,
     assert_is_instance,
@@ -82,8 +84,10 @@ __all__ = [
     "Action",
     "Store",
     "BO",
-    "ApplicationRoot",
+    "ApplicationMeta",
     "Application",
+    "ApplicationRoot",
+    "ApplicationSnapshot",
 ]
 
 
@@ -255,42 +259,6 @@ class ApplicationLock(Base):
         return type(self), ()
 
 
-class ApplicationStorage(WeakKeyDictionary):
-    """
-    Application storage.
-
-      - Holds a store for each object in the application.
-      - Can be deep copied and pickled.
-    """
-
-    def __deepcopy__(self, memo=None):
-        # type: (Optional[Dict[int, Any]]) -> ApplicationStorage
-        """
-        Make a deep copy.
-
-        :param memo: Memo dict.
-        :return: Deep copy.
-        """
-        if memo is None:
-            memo = {}
-        try:
-            deep_copy = memo[id(self)]
-        except KeyError:
-            deep_copy = memo[id(self)] = type(self)()
-            deep_copy_args = dict(self), memo
-            deep_copy.update(deepcopy(*deep_copy_args))
-        return deep_copy
-
-    def __reduce__(self):
-        # type: () -> Tuple[Type[ApplicationStorage], Tuple[Dict[BaseObject, Store]]]
-        """
-        Reduce for pickling.
-
-        :return: Class and init arguments.
-        """
-        return type(self), (dict(self),)
-
-
 class Store(InteractiveData):
     """Holds an object's state, data, metadata, hierarchy, and history information."""
 
@@ -304,9 +272,8 @@ class Store(InteractiveData):
     )  # type: DataAttribute[Optional[BaseData]]
     """Data."""
 
-    metadata = cast(
-        "DataAttribute[InteractiveDictData[str, Any]]",
-        data_dict_attribute(key_types=string_types, checked=False),
+    metadata = data_dict_attribute(
+        key_types=string_types, checked=False
     )  # type: DataAttribute[InteractiveDictData[str, Any]]
     """Metadata."""
 
@@ -370,9 +337,8 @@ class Action(Data):
     :type: objetto.bases.BaseObject
     """
 
-    locations = cast(
-        "DataAttribute[ListData[Any]]",
-        data_list_attribute(checked=False, interactive=False),
+    locations = data_protected_list_attribute(
+        checked=False
     )  # type: DataAttribute[ListData[Any]]
     """
     List of relative locations from the receiver to the sender.
@@ -393,20 +359,18 @@ class Action(Data):
 class Commit(Data):
     """Holds unmerged, modified stores."""
 
-    actions = cast(
-        "DataAttribute[ListData[Action]]",
-        data_list_attribute(Action, checked=False, finalized=True, interactive=False),
+    actions = data_protected_list_attribute(
+        Action, checked=False, finalized=True
     )  # type: Final[DataAttribute[ListData[Action]]]
     """Actions."""
 
     stores = cast(
         "DataAttribute[DictData[BaseObject, Store]]",
-        data_dict_attribute(
+        data_protected_dict_attribute(
             Store,
             checked=False,
             key_types=".._objects|BaseObject",
             key_subtypes=True,
-            interactive=False,
         ),
     )  # type: Final[DataAttribute[DictData[BaseObject, Store]]]
     """Modified stores."""
@@ -587,6 +551,7 @@ class ApplicationInternals(Base):
         "__history_cls",
         "__lock",
         "__storage",
+        "__snapshot",
         "__busy_writing",
         "__busy_hierarchy",
         "__commits",
@@ -600,7 +565,8 @@ class ApplicationInternals(Base):
         self.__app_ref = WeakReference(app)
         self.__history_cls = None  # type: Optional[Type[HistoryObject]]
         self.__lock = ApplicationLock()
-        self.__storage = ApplicationStorage()
+        self.__storage = Storage()  # type: Storage[BaseObject, Store]
+        self.__snapshot = None  # type: Optional[ApplicationSnapshot]
         self.__busy_writing = set()  # type: Set[BaseObject]
         self.__busy_hierarchy = ValueCounter()  # type: Counter[BaseObject]
         self.__commits = []  # type: List[Commit]
@@ -655,15 +621,21 @@ class ApplicationInternals(Base):
         :param obj: Object.
         :return: Store.
         """
+        if self.__snapshot is not None:
+            try:
+                return self.__snapshot._storage.query(obj)
+            except KeyError:
+                error = "object with id {} is not valid in snapshot".format(id(obj))
+                raise RuntimeError(error)
         if self.__writing:
             try:
                 return self.__commits[-1].stores[obj]
             except (IndexError, KeyError):
                 pass
         try:
-            return self.__storage[obj]
+            return self.__storage.query(obj)
         except KeyError:
-            error = "object {} is no longer valid".format(obj)
+            error = "object with id {} is no longer valid".format(id(obj))
             raise RuntimeError(error)
 
     def __read_history(
@@ -803,7 +775,9 @@ class ApplicationInternals(Base):
                     from ._changes import Batch
 
                     atomic_batch_change = Batch(
-                        name=change.name, obj=obj, metadata={"atomic_change": change}
+                        name=change.name,
+                        obj=obj,
+                        metadata={"atomic_change": change, "is_atomic": True},
                     )
                     history.__enter_batch__(atomic_batch_change)
 
@@ -1078,7 +1052,7 @@ class ApplicationInternals(Base):
                             action.receiver.__.subject.send(action, Phase.PRE)
                         )
 
-                    self.__storage.update(commit.stores)
+                    self.__storage = self.__storage.update(commit.stores)
 
                     for action in commit.actions:
                         ingest_action_exception_infos(
@@ -1159,7 +1133,15 @@ class ApplicationInternals(Base):
             except IndexError:
                 stores = InteractiveDictData()
 
-            if obj in stores or obj in self.__storage:
+            def _obj_in_storage():
+                try:
+                    self.__storage.query(obj)
+                except KeyError:
+                    return False
+                else:
+                    return True
+
+            if obj in stores or _obj_in_storage():
                 error = "object {} can't be initialized more than once".format(obj)
                 raise RuntimeError(error)
 
@@ -1197,6 +1179,21 @@ class ApplicationInternals(Base):
             commit = Commit(stores=stores)
             self.__commits.append(commit)
 
+    @contextmanager
+    def snapshot_context(self, snapshot):
+        # type: (ApplicationSnapshot) -> Iterator
+        """
+        Snapshot read context manager.
+
+        :param snapshot: Snapshot.
+        """
+        with self.read_context():
+            self.__snapshot = snapshot
+            try:
+                yield
+            finally:
+                self.__snapshot = None
+
     @contextmanager
     def read_context(self, obj=None):
         # type: (Optional[BaseObject]) -> Iterator[ReadFunction]
@@ -1451,9 +1448,53 @@ class ApplicationInternals(Base):
         """
         return self.__roots[root]
 
+    def take_snapshot(self):
+        """
+        Take a snapshot of the current application state.
+
+        :return: Application snapshot.
+        :rtype: objetto.applications.ApplicationSnapshot
+        """
+        storage = self.__storage
+        if self.__writing and self.__commits:
+            storage = storage.update(self.__commits[-1].stores)
+        app = self.__app_ref()
+        assert app is not None
+        return ApplicationSnapshot(app, storage)
+
+    @property
+    def is_writing(self):
+        # type: () -> bool
+        """
+        Whether this application is inside a write context.
+
+        :rtype: bool
+        """
+        with self.read_context():
+            return bool(self.__writing)
+
+    @property
+    def is_reading(self):
+        # type: () -> bool
+        """
+        Whether this application is inside a read context.
+
+        :rtype: bool
+        """
+        with self.read_context():
+            return len(self.__reading) > 1
+
 
 class ApplicationMeta(BaseMeta):
-    """Metaclass for `Application`."""
+    """
+    Metaclass for :class:`objetto.applications.Application`.
+
+    Inherits from:
+      - :class:`objetto.bases.BaseMeta`
+
+    Features:
+      - Check and store `root descriptors <objetto.applications.root>`_.
+    """
 
     __roots = WeakKeyDictionary(
         {}
@@ -1486,13 +1527,21 @@ class ApplicationMeta(BaseMeta):
     @property
     def _roots(cls):
         # type: () -> Mapping[str, ApplicationRoot]
-        """Attributes mapped by name."""
+        """
+        Attributes mapped by name.
+
+        :rtype: dict[str, objetto.applications.ApplicationRoot]
+        """
         return type(cls).__roots[cls]
 
     @property
     def _root_names(cls):
         # type: () -> Mapping[Any, str]
-        """Names mapped by root."""
+        """
+        Names mapped by root.
+
+        :rtype: dict[objetto.applications.ApplicationRoot, str]
+        """
         return type(cls).__root_names[cls]
 
 
@@ -1500,6 +1549,9 @@ class Application(with_metaclass(ApplicationMeta, Base)):
     """
     Application.
 
+    Metaclass:
+      - :class:`objetto.applications.ApplicationMeta`
+
     Inherits from:
       - :class:`objetto.bases.Base`
 
@@ -1530,16 +1582,30 @@ class Application(with_metaclass(ApplicationMeta, Base)):
 
     @final
     @contextmanager
-    def read_context(self):
-        # type: () -> Iterator
+    def read_context(self, snapshot=None):
+        # type: (Optional[ApplicationSnapshot]) -> Iterator
         """
         Read context.
 
+        :param snapshot: Application state snapshot.
+        :type snapshot: objetto.applications.ApplicationSnapshot
+
         :return: Context manager.
         :rtype: contextlib.AbstractContextManager
+
+        :raises ValueError: Application mismatch.
         """
-        with self.__.read_context():
-            yield
+        if snapshot is not None:
+            with ReraiseContext((TypeError, ValueError), "'snapshot' parameter"):
+                assert_is_instance(snapshot, ApplicationSnapshot)
+                if snapshot.app is not self:
+                    error = "application mismatch"
+                    raise ValueError(error)
+            with self.__.snapshot_context(snapshot):
+                yield
+        else:
+            with self.__.read_context():
+                yield
 
     @final
     @contextmanager
@@ -1571,3 +1637,98 @@ class Application(with_metaclass(ApplicationMeta, Base)):
                 raise
             else:
                 raise TemporaryContextException()
+
+    @final
+    def take_snapshot(self):
+        # type: () -> ApplicationSnapshot
+        """
+        Take a snapshot of the current application state.
+
+        :return: Application snapshot.
+        :rtype: objetto.applications.ApplicationSnapshot
+        """
+        return self.__.take_snapshot()
+
+    @property
+    def is_writing(self):
+        # type: () -> bool
+        """
+        Whether this application is inside a write context.
+
+        :rtype: bool
+        """
+        return self.__.is_writing
+
+    @property
+    def is_reading(self):
+        # type: () -> bool
+        """
+        Whether this application is inside a read context.
+
+        :rtype: bool
+        """
+        return self.__.is_reading
+
+
+@final
+class ApplicationSnapshot(Base):
+    """
+    Application snapshot.
+
+    Inherits from:
+      - :class:`objetto.bases.Base`
+
+    Features:
+      - Freezes entire application state at a moment in time.
+      - Can be used with an application's read context to travel back in time.
+
+    You can acquire a snapshot by calling the
+    :meth:`objetto.applications.Application.take_snapshot` method. You can then pass it
+    to a :meth:`objetto.applications.Application.read_context` in order to temporarily
+    bring the whole application back to the time the snapshot was taken.
+
+    .. code:: python
+
+        >>> from objetto import Application, Object, attribute
+
+        >>> class Person(Object):
+        ...     name = attribute(str)
+        ...
+        >>> app = Application()
+        >>> obj = Person(app, name="Albert")
+        >>> obj.name
+        'Albert'
+        >>> snapshot = app.take_snapshot()
+        >>> obj.name = "Einstein"
+        >>> obj.name
+        'Einstein'
+        >>> with app.read_context(snapshot):
+        ...     obj.name
+        ...
+        'Albert'
+        >>> obj.name
+        'Einstein'
+    """
+
+    __slots__ = ("__app", "__storage")
+
+    def __init__(self, app, storage):
+        # type: (Application, Storage) -> None
+        self.__app = app
+        self.__storage = storage
+
+    @property
+    def _storage(self):
+        # type: () -> Storage
+        """Internal storage."""
+        return self.__storage
+
+    @property
+    def app(self):
+        # type: () -> Application
+        """
+        Application.
+
+        :rtype: objetto.applications.Application
+        """
+        return self.__app
