@@ -7,6 +7,8 @@ from .bases import (
     FINAL_METHOD_TAG,
     INITIALIZING_TAG,
     MISSING,
+    AbstractMember,
+    AbstractMemberMeta,
     Base,
     BaseCollection,
     BaseContainer,
@@ -42,6 +44,8 @@ __all__ = [
     "make_base_cls",
     "BaseMeta",
     "Base",
+    "AbstractMemberMeta",
+    "AbstractMember",
     "abstract_member",
     "Generic",
     "BaseHashable",
