@@ -3,29 +3,38 @@
 
 from .bases import (
     BaseAuxiliaryData,
+    BaseAuxiliaryDataMeta,
     BaseData,
+    BaseDataMeta,
     BaseInteractiveAuxiliaryData,
     BaseInteractiveData,
     DataRelationship,
 )
-from .data import Data, DataAttribute, InteractiveData
-from .dict import DictData, InteractiveDictData
-from .list import InteractiveListData, ListData
-from .set import InteractiveSetData, SetData
+from .data import Data, DataAttribute, DataAttributeMeta, DataMeta, InteractiveData
+from .dict import DictData, DictDataMeta, InteractiveDictData
+from .list import InteractiveListData, ListData, ListDataMeta
+from .set import InteractiveSetData, SetData, SetDataMeta
 
 __all__ = [
     "DataRelationship",
+    "BaseDataMeta",
     "BaseData",
     "BaseInteractiveData",
+    "DataAttributeMeta",
     "DataAttribute",
+    "DataMeta",
     "Data",
     "InteractiveData",
+    "BaseAuxiliaryDataMeta",
     "BaseAuxiliaryData",
     "BaseInteractiveAuxiliaryData",
+    "DictDataMeta",
     "DictData",
     "InteractiveDictData",
+    "ListDataMeta",
     "ListData",
     "InteractiveListData",
+    "SetDataMeta",
     "SetData",
     "InteractiveSetData",
 ]
