@@ -22,6 +22,7 @@ from .._structures import (
     BaseMutableDictStructure,
     SerializationError,
 )
+from ..utils.dummy_context import DummyContext
 from .bases import (
     DELETED,
     BaseAuxiliaryObject,
@@ -50,7 +51,7 @@ if TYPE_CHECKING:
     from .._applications import Store
     from .._history import HistoryObject
 
-__all__ = ["DictObject", "MutableDictObject", "ProxyDictObject"]
+__all__ = ["DictObjectMeta", "DictObject", "MutableDictObject", "ProxyDictObject"]
 
 
 KT = TypeVar("KT")  # Key type.
@@ -112,7 +113,7 @@ class DictObjectFunctions(BaseAuxiliaryObjectFunctions):
         """
         Update values.
 
-        :param obj: Object.
+        :param obj: Dictionary object.
         :param input_values: Input values.
         :param factory: Whether to run values through factory.
         :param history: History than triggered this change during undo/redo.
@@ -120,7 +121,16 @@ class DictObjectFunctions(BaseAuxiliaryObjectFunctions):
         cls = type(obj)
         relationship = cls._relationship
         key_relationship = cls._key_relationship
-        with obj.app.__.write_context(obj) as (read, write):
+
+        # Batch context.
+        batch_name = cls._BATCH_UPDATE_NAME
+        if batch_name is None:
+            context = DummyContext()  # type: ignore
+        else:
+            context = obj._batch_context(name=batch_name)  # type: ignore
+
+        # Write context.
+        with context, obj.app.__.write_context(obj) as (read, write):
             if not input_values:
                 return
 
@@ -273,27 +283,50 @@ type.__setattr__(cast(type, DictObjectFunctions), FINAL_METHOD_TAG, True)
 
 
 class DictObjectMeta(BaseAuxiliaryObjectMeta, BaseDictStructureMeta):
-    """Metaclass for `DictObject`."""
+    """
+    Metaclass for :class:`objetto.objects.DictObject`.
+
+    Inherits from:
+      - :class:`objetto.bases.BaseAuxiliaryObjectMeta`
+      - :class:`objetto.bases.BaseDictStructureMeta`
+
+    Features:
+      - Defines a state factory.
+      - Defines a base auxiliary type.
+      - Defines a base auxiliary data type.
+    """
 
     @property
     @final
     def _state_factory(cls):
         # type: () -> Callable[..., DictState]
-        """State factory."""
+        """
+        State factory.
+
+        :rtype: type[objetto.states.DictState]
+        """
         return DictState
 
     @property
     @final
     def _base_auxiliary_type(cls):
         # type: () -> Type[DictObject]
-        """Base auxiliary object type."""
+        """
+        Base auxiliary object type.
+
+        :rtype: type[objetto.objects.DictObject]
+        """
         return DictObject
 
     @property
     @final
     def _base_auxiliary_data_type(cls):
         # type: () -> Type[DictData]
-        """Base auxiliary data type."""
+        """
+        Base auxiliary data type.
+
+        :rtype: type[objetto.data.DictData]
+        """
         return DictData
 
 
@@ -311,6 +344,9 @@ class DictObject(
     """
     Dictionary object.
 
+    Metaclass:
+      - :class:`objetto.objects.DictObjectMeta`
+
     Inherits from:
       - :class:`objetto.bases.BaseAuxiliaryObject`
       - :class:`objetto.bases.BaseDictStructure`
@@ -329,6 +365,13 @@ tuple[collections.abc.Hashable, Any]]
     __slots__ = ()
     __functions__ = DictObjectFunctions
 
+    _BATCH_UPDATE_NAME = None  # type: Optional[str]
+    """
+    Batch name for update operations.
+
+    :type: str or None
+    """
+
     def __init__(
         self,
         app,  # type: Application
@@ -512,6 +555,7 @@ tuple[collections.abc.Hashable, Any]]
                     if cls._relationship.serialized
                 )
                 self.__functions__.update(self, initial)
+            self.__post_deserialize__()
             return self
 
     @final
