@@ -22,6 +22,7 @@ from .._structures import (
     BaseMutableListStructure,
     SerializationError,
 )
+from ..utils.dummy_context import DummyContext
 from ..utils.list_operations import pre_move, resolve_continuous_slice, resolve_index
 from .bases import (
     BaseAuxiliaryObject,
@@ -53,7 +54,7 @@ if TYPE_CHECKING:
     from .._applications import Store
     from .._history import HistoryObject
 
-__all__ = ["ListObject", "MutableListObject", "ProxyListObject"]
+__all__ = ["ListObjectMeta", "ListObject", "MutableListObject", "ProxyListObject"]
 
 
 T = TypeVar("T")  # Any type.
@@ -93,7 +94,17 @@ class ListObjectFunctions(BaseAuxiliaryObjectFunctions):
         # type: (...) -> None
         cls = type(obj)
         relationship = cls._relationship
-        with obj.app.__.write_context(obj) as (read, write):
+        input_values = list(input_values)
+
+        # Batch context.
+        batch_name = cls._BATCH_INSERT_NAME
+        if batch_name is None:
+            context = DummyContext()  # type: ignore
+        else:
+            context = obj._batch_context(name=batch_name)  # type: ignore
+
+        # Write context.
+        with context, obj.app.__.write_context(obj) as (read, write):
             if not input_values:
                 return
 
@@ -210,7 +221,16 @@ class ListObjectFunctions(BaseAuxiliaryObjectFunctions):
         # type: (...) -> None
         cls = type(obj)
         relationship = cls._relationship
-        with obj.app.__.write_context(obj) as (read, write):
+
+        # Batch context.
+        batch_name = cls._BATCH_DELETE_NAME
+        if batch_name is None:
+            context = DummyContext()  # type: ignore
+        else:
+            context = obj._batch_context(name=batch_name)  # type: ignore
+
+        # Write context.
+        with context, obj.app.__.write_context(obj) as (read, write):
 
             # Get state, data, and a brand new locations cache.
             store = read()
@@ -304,7 +324,16 @@ class ListObjectFunctions(BaseAuxiliaryObjectFunctions):
         # type: (...) -> None
         cls = type(obj)
         relationship = cls._relationship
-        with obj.app.__.write_context(obj) as (read, write):
+
+        # Batch context.
+        batch_name = cls._BATCH_UPDATE_NAME
+        if batch_name is None:
+            context = DummyContext()  # type: ignore
+        else:
+            context = obj._batch_context(name=batch_name)  # type: ignore
+
+        # Write context.
+        with context, obj.app.__.write_context(obj) as (read, write):
 
             # Get state, data, and a brand new locations cache.
             store = read()
@@ -437,7 +466,17 @@ class ListObjectFunctions(BaseAuxiliaryObjectFunctions):
         history=None,  # type: Optional[HistoryObject]
     ):
         # type: (...) -> None
-        with obj.app.__.write_context(obj) as (read, write):
+        cls = type(obj)
+
+        # Batch context.
+        batch_name = cls._BATCH_MOVE_NAME
+        if batch_name is None:
+            context = DummyContext()  # type: ignore
+        else:
+            context = obj._batch_context(name=batch_name)  # type: ignore
+
+        # Write context.
+        with context, obj.app.__.write_context(obj) as (read, write):
 
             # Get state, data, and a brand new locations cache.
             store = read()
@@ -512,7 +551,18 @@ type.__setattr__(cast(type, ListObjectFunctions), FINAL_METHOD_TAG, True)
 
 
 class ListObjectMeta(BaseAuxiliaryObjectMeta, BaseListStructureMeta):
-    """Metaclass for `ListObject`."""
+    """
+    Metaclass for :class:`objetto.objects.ListObject`.
+
+    Inherits from:
+      - :class:`objetto.bases.BaseAuxiliaryObjectMeta`
+      - :class:`objetto.bases.BaseListStructureMeta`
+
+    Features:
+      - Defines a state factory.
+      - Defines a base auxiliary type.
+      - Defines a base auxiliary data type.
+    """
 
     def __init__(cls, name, bases, dct):
         # type: (str, Tuple[Type, ...], Dict[str, Any]) -> None
@@ -534,21 +584,33 @@ class ListObjectMeta(BaseAuxiliaryObjectMeta, BaseListStructureMeta):
     @final
     def _state_factory(cls):
         # type: () -> Callable[..., ListState]
-        """State factory."""
+        """
+        State factory.
+
+        :rtype: type[objetto.states.ListState]
+        """
         return ListState
 
     @property
     @final
     def _base_auxiliary_type(cls):
         # type: () -> Type[ListObject]
-        """Base auxiliary object type."""
+        """
+        Base auxiliary object type.
+
+        :rtype: type[objetto.objects.ListObject]
+        """
         return ListObject
 
     @property
     @final
     def _base_auxiliary_data_type(cls):
         # type: () -> Type[ListData]
-        """Base auxiliary data type."""
+        """
+        Base auxiliary data type.
+
+        :rtype: type[objetto.data.ListData]
+        """
         return ListData
 
 
@@ -566,6 +628,9 @@ class ListObject(
     """
     List object.
 
+    Metaclass:
+      - :class:`objetto.objects.ListObjectMeta`
+
     Inherits from:
       - :class:`objetto.bases.BaseAuxiliaryObject`
       - :class:`objetto.bases.BaseListStructure`
@@ -583,6 +648,34 @@ class ListObject(
     __slots__ = ()
     __functions__ = ListObjectFunctions
 
+    _BATCH_INSERT_NAME = None  # type: Optional[str]
+    """
+    Batch name for insert operations.
+
+    :type: str or None
+    """
+
+    _BATCH_DELETE_NAME = None  # type: Optional[str]
+    """
+    Batch name for delete operations.
+
+    :type: str or None
+    """
+
+    _BATCH_UPDATE_NAME = None  # type: Optional[str]
+    """
+    Batch name for update operations.
+
+    :type: str or None
+    """
+
+    _BATCH_MOVE_NAME = None  # type: Optional[str]
+    """
+    Batch name for move operations.
+
+    :type: str or None
+    """
+
     def __init__(self, app, initial=()):
         # type: (Application, Iterable[T]) -> None
         super(ListObject, self).__init__(app=app)
@@ -674,7 +767,10 @@ class ListObject(
         :return: Transformed.
         :rtype: objetto.objects.ListObject
         """
-        self.__functions__.insert(self, len(self._state), iterable)
+        values = list(iterable)
+        if not values:
+            return self
+        self.__functions__.insert(self, len(self._state), values)
         return self
 
     @final
@@ -851,6 +947,7 @@ class ListObject(
                     if cls._relationship.serialized
                 )
                 self.__functions__.insert(self, 0, initial)
+            self.__post_deserialize__()
             return self
 
     @final
