@@ -13,7 +13,7 @@ try:
 except ImportError:
     import collections as collections_abc  # type: ignore
 
-from six import iteritems, raise_from, with_metaclass
+from six import ensure_str, iteritems, raise_from, with_metaclass
 
 from .._applications import Application
 from .._bases import FINAL_METHOD_TAG, MISSING, Base, final, init_context, make_base_cls
@@ -26,6 +26,7 @@ from .._structures import (
     BaseAttributeStructureMeta,
     BaseMutableAttributeStructure,
 )
+from ..utils.dummy_context import DummyContext
 from ..utils.reraise_context import ReraiseContext
 from ..utils.type_checking import (
     assert_is_callable,
@@ -64,7 +65,7 @@ if TYPE_CHECKING:
     from .._history import HistoryObject
     from ..utils.factoring import LazyFactory
 
-__all__ = ["Attribute", "Object"]
+__all__ = ["AttributeMeta", "Attribute", "ObjectMeta", "Object"]
 
 
 T = TypeVar("T")  # Any type.
@@ -74,14 +75,26 @@ T = TypeVar("T")  # Any type.
 _A = TypeVar("_A", bound="Attribute")
 
 
+@final
 class AttributeMeta(BaseAttributeMeta):
-    """Metaclass for `Attribute`."""
+    """
+    Metaclass for :class:`objetto.objects.Attribute`.
+
+    Inherits from:
+      - :class:`objetto.bases.BaseAttributeMeta`
+
+    Features:
+      - Defines relationship type.
+    """
 
     @property
-    @final
     def _relationship_type(cls):
         # type: () -> Type[Relationship]
-        """Relationship type."""
+        """
+        Relationship type.
+
+        :rtype: type[objetto.objects.Relationship]
+        """
         return Relationship
 
 
@@ -90,6 +103,9 @@ class Attribute(with_metaclass(AttributeMeta, BaseAttribute[T])):
     """
     Attribute descriptor for :class:`objetto.objects.Object` classes.
 
+    Metaclass:
+      - :class:`objetto.objects.AttributeMeta`
+
     Inherits from:
       - :class:`objetto.bases.BaseAttribute`
 
@@ -119,6 +135,8 @@ class Attribute(with_metaclass(AttributeMeta, BaseAttribute[T])):
     :param abstracted: If True, attribute needs to be overridden by subclasses.
     :type abstracted: bool
 
+    :param metadata: Metadata.
+
     :param delegated: Whether attribute allows for delegates to be defined.
     :type delegated: bool
 
@@ -129,6 +147,9 @@ objetto.objects.Attribute or None
     :param deserialize_to: Non-serialized attribute to deserialize this into.
     :type deserialize_to: objetto.objects.Attribute or None
 
+    :param batch_name: Batch name.
+    :type batch_name: str or None
+
     :raises TypeError: Invalid parameter type.
     :raises ValueError: Invalid parameter value.
     :raises ValueError: Can't declare same dependency more than once.
@@ -147,6 +168,7 @@ objetto.objects.Attribute or None
         "__fset",
         "__fdel",
         "__data_attribute",
+        "__batch_name",
     )
 
     def __init__(
@@ -160,9 +182,11 @@ objetto.objects.Attribute or None
         deletable=None,  # type: Optional[bool]
         finalized=False,  # type: bool
         abstracted=False,  # type: bool
+        metadata=None,  # type: Any
         delegated=False,  # type: bool
         dependencies=None,  # type: Optional[Union[Iterable[Attribute], Attribute]]
         deserialize_to=None,  # type: Optional[Attribute]
+        batch_name=None,  # type: Optional[str]
     ):
         # type: (...) -> None
 
@@ -233,6 +257,7 @@ objetto.objects.Attribute or None
             deletable=deletable,
             finalized=finalized,
             abstracted=abstracted,
+            metadata=metadata,
         )
 
         self.__delegated = bool(delegated)
@@ -242,6 +267,9 @@ objetto.objects.Attribute or None
         self.__fset = None  # type: Optional[Callable]
         self.__fdel = None  # type: Optional[Callable]
         self.__data_attribute = None  # type: Optional[DataAttribute]
+        self.__batch_name = (
+            ensure_str(batch_name) if batch_name is not None else None
+        )  # type: Optional[str]
 
     def __set__(self, instance, value):
         # type: (Object, T) -> None
@@ -282,6 +310,7 @@ objetto.objects.Attribute or None
                 "fget": self.fget,
                 "fset": self.fset,
                 "fdel": self.fdel,
+                "batch_name": self.batch_name,
             }
         )
         return dct
@@ -466,6 +495,26 @@ objetto.objects.Attribute or None
         """
         return self.__fdel
 
+    @property
+    def constant(self):
+        # type: () -> bool
+        """
+        Whether attribute is constant.
+
+        :rtype: bool
+        """
+        return super(Attribute, self).constant and not self.delegated
+
+    @property
+    def batch_name(self):
+        # type: () -> Optional[str]
+        """
+        Batch name.
+
+        :rtype: str or None
+        """
+        return self.__batch_name
+
     @property
     def data_attribute(self):
         # type: () -> Optional[DataAttribute]
@@ -487,6 +536,7 @@ objetto.objects.Attribute or None
                     deletable=True,
                     finalized=False,
                     abstracted=False,
+                    metadata=self.metadata,
                 )
         return self.__data_attribute
 
@@ -567,9 +617,10 @@ class Functions(BaseObjectFunctions):
             n for n, a in iteritems(cls._attributes) if not a.required
         )  # type: Set[str]
         if missing_attributes.difference(optional_attributes):
-            error = "missing required attribute{} {}".format(
+            error = "missing required attribute{} {} in '{}' object".format(
                 "s" if len(missing_attributes) != 1 else "",
                 ", ".join("'{}'".format(n) for n in missing_attributes),
+                cls.__fullname__,
             )
             raise TypeError(error)
 
@@ -777,7 +828,16 @@ type.__setattr__(cast(type, Functions), FINAL_METHOD_TAG, True)
 
 class ObjectMeta(BaseAttributeStructureMeta, BaseObjectMeta):
     """
-    Metaclass for `Object`.
+    Metaclass for :class:`objetto.objects.Object`.
+
+    Inherits from:
+      - :class:`objetto.bases.BaseAttributeStructureMeta`
+      - :class:`objetto.bases.BaseObjectMeta`
+
+    Features:
+      - Support for :class:`objetto.objects.Attribute` descriptors.
+      - Compute and store attribute dependencies.
+      - Constructs automatic `Data` class.
 
     :raises TypeError: Attribute is delegated but no delegates were defined.
     :raises TypeError: Attribute declares a dependency which is not available.
@@ -862,63 +922,99 @@ class ObjectMeta(BaseAttributeStructureMeta, BaseObjectMeta):
     @final
     def _attribute_type(cls):
         # type: () -> Type[Attribute]
-        """Attribute type."""
+        """
+        Attribute type.
+
+        :rtype: type[objetto.objects.Attribute]
+        """
         return Attribute
 
     @property
     @final
     def _attributes(cls):
         # type: () -> Mapping[str, Attribute]
-        """Attributes mapped by name."""
+        """
+        Attributes mapped by name.
+
+        :rtype: dict[str, objetto.objects.Attribute]
+        """
         return cast("Mapping[str, Attribute]", super(ObjectMeta, cls)._attributes)
 
     @property
     @final
     def _attribute_names(cls):
         # type: () -> Mapping[Attribute, str]
-        """Names mapped by attribute."""
+        """
+        Names mapped by attribute.
+
+        :rtype: dict[objetto.objects.Attribute, str]
+        """
         return cast("Mapping[Attribute, str]", super(ObjectMeta, cls)._attribute_names)
 
     @property
     @final
     def _state_factory(cls):
         # type: () -> Callable[..., DictState]
-        """State factory."""
+        """
+        State factory.
+
+        :rtype: type[objetto.states.DictState]
+        """
         return DictState
 
     @property
     @final
     def _attribute_dependencies(cls):
         # type: () -> DictState[str, SetState[str]]
-        """Attribute dependencies."""
+        """
+        Attribute dependencies.
+
+        :rtype: dict[str, set[str]]
+        """
         return type(cls).__attribute_dependencies[cls]
 
     @property
     @final
     def _attribute_dependents(cls):
         # type: () -> DictState[str, SetState[str]]
-        """Attribute dependents."""
+        """
+        Attribute dependents.
+
+        :rtype: dict[str, set[str]]
+        """
         return type(cls).__attribute_dependents[cls]
 
     @property
     @final
     def _attribute_flattened_dependencies(cls):
         # type: () -> DictState[str, SetState[str]]
-        """Flattened attribute dependencies."""
+        """
+        Flattened attribute dependencies.
+
+        :rtype: dict[str, set[str]]
+        """
         return type(cls).__attribute_flattened_dependencies[cls]
 
     @property
     @final
     def _attribute_flattened_dependents(cls):
         # type: () -> DictState[str, SetState[str]]
-        """Flattened attribute dependents."""
+        """
+        Flattened attribute dependents.
+
+        :rtype: dict[str, set[str]]
+        """
         return type(cls).__attribute_flattened_dependents[cls]
 
     @property
     @final
     def Data(cls):
         # type: () -> Type[Data]
-        """Data type."""
+        """
+        Data type.
+
+        :rtype: type[objetto.data.Data]
+        """
 
         # Try to get cached data type.
         mcs = type(cls)
@@ -986,6 +1082,9 @@ class Object(
     """
     Object.
 
+    Metaclass:
+      - :class:`objetto.objects.ObjectMeta`
+
     Inherits from:
       - :class:`objetto.bases.BaseMutableObject`
       - :class:`objetto.bases.BaseMutableAttributeStructure`
@@ -1120,8 +1219,15 @@ class Object(
 
         :raises AttributeError: Attribute is not changeable and already has a value.
         """
-        self.__functions__.update(self, {name: value})
-        return self
+        attribute = self._get_attribute(name)
+        batch_name = attribute.batch_name
+        if batch_name is None:
+            context = DummyContext()  # type: ignore
+        else:
+            context = self._batch_context(name=batch_name)  # type: ignore
+        with context:
+            self.__functions__.update(self, {name: value})
+            return self
 
     @final
     def _delete(self, name):
@@ -1138,8 +1244,15 @@ class Object(
         :raises KeyError: Attribute does not exist or has no value.
         :raises AttributeError: Attribute is not deletable.
         """
-        self.__functions__.update(self, {name: DELETED})
-        return self
+        attribute = self._get_attribute(name)
+        batch_name = attribute.batch_name
+        if batch_name is None:
+            context = DummyContext()  # type: ignore
+        else:
+            context = self._batch_context(name=batch_name)  # type: ignore
+        with context:
+            self.__functions__.update(self, {name: DELETED})
+            return self
 
     @final
     def _locate(self, child):
@@ -1244,6 +1357,7 @@ class Object(
                     factory=False,
                 )
                 Functions.check_missing(cls, self._state)
+            self.__post_deserialize__()
             return self
 
     @final
