@@ -22,6 +22,7 @@ from .._structures import (
     BaseSetStructureMeta,
     SerializationError,
 )
+from ..utils.dummy_context import DummyContext
 from .bases import (
     BaseAuxiliaryObject,
     BaseAuxiliaryObjectFunctions,
@@ -47,7 +48,7 @@ if TYPE_CHECKING:
     from .._applications import Store
     from .._history import HistoryObject
 
-__all__ = ["SetObject", "MutableSetObject", "ProxySetObject"]
+__all__ = ["SetObjectMeta", "SetObject", "MutableSetObject", "ProxySetObject"]
 
 
 T = TypeVar("T")  # Any type.
@@ -97,7 +98,17 @@ class SetObjectFunctions(BaseAuxiliaryObjectFunctions):
         # type: (...) -> None
         cls = type(obj)
         relationship = cls._relationship
-        with obj.app.__.write_context(obj) as (read, write):
+        input_values = set(input_values)
+
+        # Batch context.
+        batch_name = cls._BATCH_UPDATE_NAME
+        if batch_name is None:
+            context = DummyContext()  # type: ignore
+        else:
+            context = obj._batch_context(name=batch_name)  # type: ignore
+
+        # Write context.
+        with context, obj.app.__.write_context(obj) as (read, write):
             if not input_values:
                 return
 
@@ -217,7 +228,16 @@ class SetObjectFunctions(BaseAuxiliaryObjectFunctions):
         # type: (...) -> None
         cls = type(obj)
         relationship = cls._relationship
-        with obj.app.__.write_context(obj) as (read, write):
+
+        # Batch context.
+        batch_name = cls._BATCH_REMOVE_NAME
+        if batch_name is None:
+            context = DummyContext()  # type: ignore
+        else:
+            context = obj._batch_context(name=batch_name)  # type: ignore
+
+        # Write context.
+        with context, obj.app.__.write_context(obj) as (read, write):
             if not input_values:
                 return
 
@@ -303,27 +323,50 @@ type.__setattr__(cast(type, SetObjectFunctions), FINAL_METHOD_TAG, True)
 
 
 class SetObjectMeta(BaseAuxiliaryObjectMeta, BaseSetStructureMeta):
-    """Metaclass for `SetObject`."""
+    """
+    Metaclass for :class:`objetto.objects.SetObject`.
+
+    Inherits from:
+      - :class:`objetto.bases.BaseAuxiliaryObjectMeta`
+      - :class:`objetto.bases.BaseSetStructureMeta`
+
+    Features:
+      - Defines a state factory.
+      - Defines a base auxiliary type.
+      - Defines a base auxiliary data type.
+    """
 
     @property
     @final
     def _state_factory(cls):
         # type: () -> Callable[..., SetState]
-        """State factory."""
+        """
+        State factory.
+
+        :rtype: type[objetto.states.SetState]
+        """
         return SetState
 
     @property
     @final
     def _base_auxiliary_type(cls):
         # type: () -> Type[SetObject]
-        """Base auxiliary object type."""
+        """
+        Base auxiliary object type.
+
+        :rtype: type[objetto.objects.SetObject]
+        """
         return SetObject
 
     @property
     @final
     def _base_auxiliary_data_type(cls):
         # type: () -> Type[SetData]
-        """Base auxiliary data type."""
+        """
+        Base auxiliary data type.
+
+        :rtype: type[objetto.data.SetData]
+        """
         return SetData
 
 
@@ -341,6 +384,9 @@ class SetObject(
     """
     Set object.
 
+    Metaclass:
+      - :class:`objetto.objects.SetObjectMeta`
+
     Inherits from:
       - :class:`objetto.bases.BaseAuxiliaryObject`
       - :class:`objetto.bases.BaseSetStructure`
@@ -358,6 +404,20 @@ class SetObject(
     __slots__ = ()
     __functions__ = SetObjectFunctions
 
+    _BATCH_UPDATE_NAME = None  # type: Optional[str]
+    """
+    Batch name for update operations.
+
+    :type: str or None
+    """
+
+    _BATCH_REMOVE_NAME = None  # type: Optional[str]
+    """
+    Batch name for remove operations.
+
+    :type: str or None
+    """
+
     @classmethod
     @final
     def _from_iterable(cls, iterable):
@@ -575,6 +635,7 @@ class SetObject(
                     if cls._relationship.serialized
                 )
                 self.__functions__.update(self, initial)
+            self.__post_deserialize__()
             return self
 
     @final
