@@ -30,7 +30,7 @@ from ..utils.custom_repr import custom_iterable_repr, custom_mapping_repr
 from ..utils.factoring import format_factory, import_factory, run_factory
 from ..utils.recursive_repr import recursive_repr
 from ..utils.reraise_context import ReraiseContext
-from ..utils.type_checking import assert_is_instance
+from ..utils.type_checking import assert_is_instance, get_type_names
 from .bases import (
     BaseInteractiveStructure,
     BaseMutableStructure,
@@ -50,6 +50,7 @@ if TYPE_CHECKING:
         Optional,
         Tuple,
         Type,
+        Union,
     )
 
     from ..utils.factoring import LazyFactory
@@ -68,13 +69,29 @@ T = TypeVar("T")  # Any type.
 
 
 class BaseAttributeMeta(BaseMeta):
-    """Metaclass for `BaseAttribute`."""
+    """
+    Metaclass for :class:`objetto.bases.BaseAttribute`.
+
+    Inherits from:
+      - :class:`objetto.bases.BaseMeta`
+
+    Inherited by:
+      - :class:`objetto.data.DataAttributeMeta`
+      - :class:`objetto.objects.AttributeMeta`
+
+    Features:
+      - Defines relationship type.
+    """
 
     @property
     @abstractmethod
     def _relationship_type(cls):
         # type: () -> Type[BaseRelationship]
-        """Relationship type."""
+        """
+        Relationship type.
+
+        :rtype: type[objetto.bases.BaseRelationship]
+        """
         return BaseRelationship
 
 
@@ -86,6 +103,9 @@ class BaseAttribute(with_metaclass(BaseAttributeMeta, BaseHashable, Generic[T]))
     """
     Base attribute descriptor for :class:`objetto.bases.BaseAttributeStructure` classes.
 
+    Metaclass:
+      - :class:`objetto.bases.BaseAttributeMeta`
+
     Inherits from:
       - :class:`objetto.bases.BaseHashable`
       - :class:`typing.Generic`
@@ -120,6 +140,8 @@ class BaseAttribute(with_metaclass(BaseAttributeMeta, BaseHashable, Generic[T]))
     :param abstracted: If True, attribute needs to be overridden by subclasses.
     :type abstracted: bool
 
+    :param metadata: Metadata.
+
     :raises TypeError: Invalid parameter type.
     :raises ValueError: Invalid parameter value.
     :raises ValueError: Can't specify both 'default' and 'default_factory' arguments.
@@ -137,6 +159,7 @@ class BaseAttribute(with_metaclass(BaseAttributeMeta, BaseHashable, Generic[T]))
         "_deletable",
         "__finalized",
         "__abstracted",
+        "__metadata",
         ABSTRACT_TAG,
         FINAL_METHOD_TAG,
     )
@@ -152,6 +175,7 @@ class BaseAttribute(with_metaclass(BaseAttributeMeta, BaseHashable, Generic[T]))
         deletable=False,  # type: bool
         finalized=False,  # type: bool
         abstracted=False,  # type: bool
+        metadata=None,  # type: Any
     ):
         # type: (...) -> None
         cls = type(self)
@@ -194,6 +218,7 @@ class BaseAttribute(with_metaclass(BaseAttributeMeta, BaseHashable, Generic[T]))
         self._deletable = bool(deletable)
         self.__finalized = bool(finalized)
         self.__abstracted = bool(abstracted)
+        self.__metadata = metadata
 
         if finalized:
             setattr(self, FINAL_METHOD_TAG, True)
@@ -202,7 +227,7 @@ class BaseAttribute(with_metaclass(BaseAttributeMeta, BaseHashable, Generic[T]))
 
     @overload
     def __get__(self, instance, owner):
-        # type: (_BA, None, Type[BaseAttributeStructure]) -> _BA
+        # type: (_BA, None, Type[BaseAttributeStructure]) -> Union[_BA, T]
         pass
 
     @overload
@@ -217,8 +242,8 @@ class BaseAttribute(with_metaclass(BaseAttributeMeta, BaseHashable, Generic[T]))
 
     def __get__(self, instance, owner):
         """
-        Get attribute value when accessing from valid instance or this descriptor
-        otherwise.
+        Get attribute value when accessing from valid instance or when attribute is
+        constant. Get this descriptor otherwise.
 
         :param instance: Instance.
         :type instance: objetto.bases.BaseAttributeStructure or None
@@ -233,7 +258,10 @@ class BaseAttribute(with_metaclass(BaseAttributeMeta, BaseHashable, Generic[T]))
             attribute_type = getattr(type(instance), "_attribute_type", None)
             if attribute_type is not None and isinstance(self, attribute_type):
                 return self.get_value(instance)
-        return self
+        elif self.constant:
+            return self.default
+        else:
+            return self
 
     @final
     def __hash__(self):
@@ -295,6 +323,7 @@ class BaseAttribute(with_metaclass(BaseAttributeMeta, BaseHashable, Generic[T]))
             "deletable": self.deletable,
             "finalized": self.finalized,
             "abstracted": self.abstracted,
+            "metadata": self.metadata,
         }
 
     def get_name(self, instance):
@@ -444,6 +473,12 @@ class BaseAttribute(with_metaclass(BaseAttributeMeta, BaseHashable, Generic[T]))
         """
         return self.__abstracted
 
+    @property
+    def metadata(self):
+        # type: () -> Any
+        """Metadata."""
+        return self.__metadata
+
     @property
     def has_default(self):
         # type: () -> bool
@@ -454,9 +489,35 @@ class BaseAttribute(with_metaclass(BaseAttributeMeta, BaseHashable, Generic[T]))
         """
         return self.default is not MISSING or self.default_factory is not None
 
+    @property
+    def constant(self):
+        # type: () -> bool
+        """
+        Whether attribute is constant.
+
+        :rtype: bool
+        """
+        return (
+            self.default is not MISSING
+            and not self.changeable
+            and self.relationship.factory is None
+        )
+
 
 class BaseAttributeStructureMeta(BaseStructureMeta):
-    """Metaclass for `BaseAttributeStructure`."""
+    """
+    Metaclass for :class:`objetto.bases.BaseAttributeStructure`.
+
+    Inherits from:
+      - :class:`objetto.bases.BaseStructureMeta`
+
+    Inherited by:
+      - :class:`objetto.data.DataMeta`
+      - :class:`objetto.objects.ObjectMeta`
+
+    Features:
+      - Support for :class:`objetto.bases.BaseAttribute` descriptors.
+    """
 
     __attributes = WeakKeyDictionary(
         {}
@@ -475,13 +536,132 @@ class BaseAttributeStructureMeta(BaseStructureMeta):
         except NotImplementedError:
             attribute_type = None
 
-        # Store attributes.
-        attributes = {}
+        # Store attributes, check implementation of abstract attributes along the way.
+        attributes = {}  # type: Dict[str, BaseAttribute]
+        abstract_attributes = {}  # type: Dict[str, Tuple[Type, BaseAttribute]]
         if attribute_type is not None:
             for base in reversed(getmro(cls)):
                 for member_name, member in iteritems(base.__dict__):
+
+                    # Implementing checked abstract attribute.
+                    if (
+                        member_name in abstract_attributes and
+                        abstract_attributes[member_name][1].relationship.checked
+                    ):
+                        abstract_base, abstract_attribute = abstract_attributes[
+                            member_name
+                        ]
+
+                        # Not a supported attribute.
+                        if not isinstance(member, attribute_type):
+                            error = (
+                                "can't implement abstract attribute '{}.{}' in '{}.{}' "
+                                "with unsupported member type '{}', expected '{}'"
+                            ).format(
+                                abstract_base.__name__,
+                                member_name,
+                                name,
+                                member_name,
+                                type(member).__name__,
+                                attribute_type.__fullname__,
+                            )
+                            raise TypeError(error)
+
+                        # Check types if possible.
+                        for typ in abstract_attribute.relationship.types:
+
+                            # No types declared in implementation.
+                            if not member.relationship.types:
+
+                                error = (
+                                    "abstract attribute '{}.{}' declares types, but "
+                                    "implementation in '{}.{}' does not"
+                                ).format(
+                                    abstract_base.__name__,
+                                    member_name,
+                                    name,
+                                    member_name,
+                                )
+                                raise TypeError(error)
+
+                            # Types are incompatible (skip lazy imported).
+                            if not any(
+                                isinstance(typ, string_types)
+                                or isinstance(t, string_types)
+                                or issubclass(t, typ)
+                                for t in member.relationship.types
+                            ):
+                                error = (
+                                    "types in attribute '{}.{}' {} are not compatible "
+                                    "with abstract '{}.{}' attribute types {}"
+                                ).format(
+                                    name,
+                                    member_name,
+                                    get_type_names(member.relationship.types),
+                                    abstract_base.__name__,
+                                    member_name,
+                                    get_type_names(
+                                        abstract_attribute.relationship.types
+                                    ),
+                                )
+                                raise TypeError(error)
+
+                        # Check other parameters.
+                        if abstract_attribute.changeable and not member.changeable:
+                            error = (
+                                "abstract attribute '{}.{}' is changeable, but "
+                                "implementation in '{}.{}' is not"
+                            ).format(
+                                abstract_base.__name__,
+                                member_name,
+                                name,
+                                member_name,
+                            )
+                            raise TypeError(error)
+                        if not abstract_attribute.changeable and member.changeable:
+                            error = (
+                                "abstract attribute '{}.{}' is not changeable, but "
+                                "implementation in '{}.{}' is"
+                            ).format(
+                                abstract_base.__name__,
+                                member_name,
+                                name,
+                                member_name,
+                            )
+                            raise TypeError(error)
+
+                        if abstract_attribute.deletable and not member.deletable:
+                            error = (
+                                "abstract attribute '{}.{}' is deletable, but "
+                                "implementation in '{}.{}' is not"
+                            ).format(
+                                abstract_base.__name__,
+                                member_name,
+                                name,
+                                member_name,
+                            )
+                            raise TypeError(error)
+                        if not abstract_attribute.deletable and member.deletable:
+                            error = (
+                                "abstract attribute '{}.{}' is not deletable, but "
+                                "implementation in '{}.{}' is"
+                            ).format(
+                                abstract_base.__name__,
+                                member_name,
+                                name,
+                                member_name,
+                            )
+                            raise TypeError(error)
+
+                    # Store attribute.
                     if isinstance(member, attribute_type):
                         attributes[member_name] = member
+
+                        # Abstract attribute, remember it.
+                        if member.abstracted:
+                            abstract_attributes[member_name] = base, member
+
+                    # Not an attribute.
                     elif member_name in attributes:
                         del attributes[member_name]
 
@@ -498,19 +678,31 @@ class BaseAttributeStructureMeta(BaseStructureMeta):
     @abstractmethod
     def _attribute_type(cls):
         # type: () -> Type[BaseAttribute]
-        """Attribute type."""
+        """
+        Attribute type.
+
+        :rtype: type[objetto.bases.BaseAttribute]
+        """
         raise NotImplementedError()
 
     @property
     def _attributes(cls):
         # type: () -> Mapping[str, BaseAttribute]
-        """Attributes mapped by name."""
+        """
+        Attributes mapped by name.
+
+        :rtype: dict[str, objetto.bases.BaseAttribute]
+        """
         return type(cls).__attributes[cls]
 
     @property
     def _attribute_names(cls):
         # type: () -> Mapping[Any, str]
-        """Names mapped by attribute."""
+        """
+        Names mapped by attribute.
+
+        :rtype: dict[objetto.bases.BaseAttribute, str]
+        """
         return type(cls).__attribute_names[cls]
 
 
@@ -524,6 +716,9 @@ class BaseAttributeStructure(
     """
     Base attribute structure.
 
+    Metaclass:
+      - :class:`objetto.bases.BaseAttributeStructureMeta`
+
     Inherits from:
       - :class:`objetto.bases.BaseStructure`
 
@@ -580,6 +775,8 @@ class BaseAttributeStructure(
             prefix="{}(".format(type(self).__fullname__),
             template="{key}={value}",
             suffix=")",
+            sorting=True,
+            sort_key=lambda p: p[0],
             key_repr=str,
         )
 
